import {DragDropContext, Draggable, DraggableProvided, DroppableProvided, DropResult} from 'react-beautiful-dnd';
import {StrictModeDroppable} from "./StrictModeDroppable";
import {Dispatch, SetStateAction, useEffect, useState} from "react";
import {Texts} from "../hooks/useAnimate";
import {useRecoilValue} from "recoil";
import {languageAtom} from "../atoms/languageAtom";
import {glucoseTimeline, InsulinDragDropTitle} from "../../public/lit/texts";
import {useTranslate} from "../hooks/useTranslate";

interface GlucoseTimelineItem {
    id: string;
    text: {
        EN: string;
        SK: string;
    };
    image: string;
}

interface InsulinDragTestProps {
    handleNext: () => void,
    setScore: Dispatch<SetStateAction<number>>,
    score: number
}
const InsulinDragNDropTest = ({ handleNext, setScore, score }: InsulinDragTestProps) => {
    const title = useTranslate(InsulinDragDropTitle);
    const shuffleArray = (array: GlucoseTimelineItem[]) => {
        for (let i = array.length - 1; i > 0; i--) {
            const j = Math.floor(Math.random() * (i + 1));
            [array[i], array[j]] = [array[j], array[i]];
        }
        return array;
    };

    const initialOrder = shuffleArray([...glucoseTimeline]);

    const [timeline, setTimeline] = useState(initialOrder);
    const language = useRecoilValue(languageAtom);
    const [isCorrectOrder, setIsCorrectOrder] = useState(false);

    const handleOnDragEnd = (result: DropResult) => {
        console.log(result);
        if (!result.destination) {
            return;
        }
        const items = Array.from(timeline);
        const [reorderedItem] = items.splice(result.source.index, 1);
        items.splice(result.destination.index, 0, reorderedItem);
        setTimeline(items);
    }

    useEffect(() => {
        const isOrderCorrect = timeline.every((item, index) => item.id === glucoseTimeline[index].id);
        setIsCorrectOrder(isOrderCorrect);
    }, [timeline]);

    const handleNextClick = () => {
        setScore(score + 1);
        handleNext();
    };

    return (
        <>
            <p className="text-lg">{title}</p>
            <div className="w-1/3 inline-block mt-4">
                <DragDropContext onDragEnd={handleOnDragEnd}>
                    <StrictModeDroppable droppableId="glucoseTimeline">
                        {(provided: DroppableProvided) => (
                            <ul className={`glucoseTimeline ${isCorrectOrder ? 'correct' : ''}`} {...provided.droppableProps}
                                ref={provided.innerRef}>
                                {timeline.map(({id, text, image}, index) => {
                                    return (
                                        <Draggable key={id} draggableId={id} index={index}>
                                            {(provided: DraggableProvided) => (
                                                <li ref={provided.innerRef} {...provided.draggableProps} {...provided.dragHandleProps}>
                                                    <div className="timeline-image">
                                                        <img src={image}
                                                             alt={`${text[language as keyof Texts]} image`}/>
                                                    </div>
                                                    <p className="text-lg">
                                                        {text[language as keyof Texts]}
                                                    </p>
                                                </li>
                                            )}
                                        </Draggable>
                                    );
                                })}
                                {provided.placeholder}
                            </ul>
                        )}
                    </StrictModeDroppable>
                </DragDropContext>
                {isCorrectOrder && <p className="mb-4 bg-green-main">CORRECT!</p>}
                {isCorrectOrder && <button className="border-4 border-black font-primary bg-yellow-main"
                                           onClick={handleNextClick}>Next</button>}
            </div>
        </>
    )
}

export default InsulinDragNDropTest;
