import {useEffect, useRef, useState} from "react";
import {
    correct,
    finishButton,
    insulinTestQuestions,
    nextButton,
    stepperLabels,
    summaryHappy,
    summarySad,
    yourScore
} from "../../public/lit/texts";
import {Stepper} from "react-stepa";
import {useRecoilValue} from "recoil";
import {languageAtom} from "../atoms/languageAtom";
import {Texts, useAnimate} from "../hooks/useAnimate";
import {useTranslate} from "../hooks/useTranslate";
import {useNavigate} from "react-router-dom";
import InsulinDragNDropTest from "./InsulinDragNDropTest";
import {AnimateCC} from "react-adobe-animate";
import Typewriter, {TypewriterClass} from "typewriter-effect";

const InsulinTest = () => {
    const language = useRecoilValue(languageAtom);
    const [currentStep, setCurrentStep] = useState(0);
    const nextButtonText = useTranslate(nextButton);
    const finishButtonText = useTranslate(finishButton);
    const correctButtonText = useTranslate(correct);
    const scoreText = useTranslate(yourScore);
    const [selectedAnswer, setSelectedAnswer] = useState<number | null>(null);
    const [isAnswered, setIsAnswered] = useState(false);
    const [score, setScore] = useState(0);
    const [isScoreBelow50, setIsScoreBelow50] = useState((score / 5) * 100 < 50);
    const navigate = useNavigate();
    const typewriterRef = useRef<TypewriterClass>();
    const {getAnimationObject} = useAnimate({
        typewriterRef,
        link: '/',
        text: [],
        audioFiles: []
    });

    useEffect(() => {
        setIsScoreBelow50((score / 5) * 100 < 50);
    }, [score]);

    const handleNextStep = () => {
        setCurrentStep(currentStep + 1);
        setSelectedAnswer(null);
        setIsAnswered(false);
    };

    const handleAnswerClick = (index: number) => {
        if (!isAnswered) {
            setSelectedAnswer(index);
            setIsAnswered(true);
            if (insulinTestQuestions[currentStep].answers[index].isCorrect) {
                setScore(score + 1);
            }
        }
    };

    const generateSteps = () => {
        const updatedSteps = stepperLabels.map((question) => ({
            label: question[language as keyof Texts],
            description: "",
        }));

        updatedSteps.splice(updatedSteps.length - 1, 0, {label: "Glucose Timeline", description: ""});
        return updatedSteps;
    };

    const isDraggableTestStep = currentStep === 4;
    const isLastStep = currentStep === 6;
    const isSummaryStep = currentStep === 5;

    return (
        <div className="container mx-auto py-8">
            <Stepper
                customStyle={{
                    completed: '#3D348B',
                    pending: '#aaccff',
                    progress: '#5599ff',
                }}
                steps={generateSteps()}
                activeStep={currentStep}
            />
            <div className="text-center">
                {isSummaryStep ? (
                    <>
                        <p className="text-xl font-semibold">
                            {`${scoreText} ${(score / 5) * 100}%`}
                        </p>
                        {isScoreBelow50 ? (
                            <div id="intro-text" className="font-speech">
                                <Typewriter
                                    options={{
                                        delay: 40,
                                    }}
                                    onInit={(typewriter) => {
                                        typewriter.typeString(summarySad[language as keyof Texts])
                                        typewriter.start()
                                    }}
                                />
                            </div>
                        ) : (
                            <div id="intro-text" className="font-speech text-center">
                                <Typewriter
                                    options={{
                                        delay: 40,
                                    }}
                                    onInit={(typewriter) => {
                                        typewriter.typeString(summaryHappy[language as keyof Texts])
                                        typewriter.start()
                                    }}
                                />
                            </div>
                        )}
                        <div className="w-[300px] max-h-[700px] h-auto justify-center inline-block">
                            {isScoreBelow50 ? (
                                <div className="w-[300px] max-h-[700px] h-auto">
                                    <AnimateCC
                                        animationName="sad"
                                        getAnimationObject={getAnimationObject}
                                    />
                                </div>
                            ) : (
                                <div className="w-[300px] max-h-[700px] h-auto">
                                    <AnimateCC
                                        animationName="happy"
                                        getAnimationObject={getAnimationObject}
                                    />
                                </div>
                            )}
                        </div>
                    </>
                ) : isDraggableTestStep ? (
                    <></>
                ) : (
                    <p className="text-xl font-semibold py-4">
                        {language === "EN"
                            ? insulinTestQuestions[currentStep].EN
                            : insulinTestQuestions[currentStep].SK}
                    </p>
                )}
            </div>
            <div className="text-center mt-4">
                {isSummaryStep ? (
                    <button
                        className="bg-primary-light hover:bg-primary-main text-white font-bold py-2 px-4 rounded"
                        onClick={() => navigate("/symptoms")}
                    >
                        {nextButtonText}
                    </button>
                ) : isDraggableTestStep ? (
                    <InsulinDragNDropTest handleNext={handleNextStep} setScore={setScore} score={score}/>
                ) : (
                    <div>
                        <div>
                            <div className="ml-4">
                                {insulinTestQuestions[currentStep].answers.map((answer, index) => (
                                    <button
                                        key={index}
                                        className={` ${
                                            selectedAnswer === index
                                                ? answer.isCorrect
                                                    ? "bg-green-main"
                                                    : "bg-red-500"
                                                : "bg-primary-dark"
                                        } ${
                                            selectedAnswer !== null ? "cursor-not-allowed" : ""
                                        } hover:bg-primary-light font-bold py-2 px-4 rounded mr-4 mt-4`}
                                        onClick={() => handleAnswerClick(index)}
                                        disabled={isAnswered}
                                    >
                                        {language === "EN" ? answer.EN : answer.SK}
                                    </button>
                                ))}
                            </div>
                            {selectedAnswer !== null && (
                                <div
                                    className={`text-${insulinTestQuestions[currentStep].answers[selectedAnswer].isCorrect ? "green" : "red"}-500 mt-4`}>
                                    {insulinTestQuestions[currentStep].answers[selectedAnswer].isCorrect
                                        ? `${correctButtonText}`
                                        : insulinTestQuestions[currentStep].explanation[language as keyof Texts]}
                                </div>
                            )}
                            {!isLastStep && (
                                <button
                                    className={`${!isAnswered ? 'bg-gray-main' : 'bg-primary-light  hover:bg-primary-main'} text-white font-bold mt-4 py-2 px-4 rounded`}
                                    onClick={handleNextStep}
                                    disabled={!isAnswered}
                                >
                                    {nextButtonText}
                                </button>
                            )}
                            {isLastStep && (
                                <button
                                    className={`${!isAnswered ? 'bg-gray-main' : 'bg-primary-light  hover:bg-primary-main'} text-white font-bold mt-4 py-2 px-4 rounded`}
                                    onClick={() => {
                                        setCurrentStep(currentStep + 1);
                                    }}
                                    disabled={!isAnswered}
                                >
                                    {finishButtonText}
                                </button>
                            )}
                        </div>
                        <div className="w-96 h-96 mt-8 inline-block border-sky-500 border-4 border-double">
                            <AnimateCC
                                animationName={`insulinTest_q${currentStep + 1}`}
                                getAnimationObject={getAnimationObject}
                            />
                        </div>
                    </div>
                )}
            </div>

        </div>
    );
};
export default InsulinTest;
